from .grpc_lib.darsman_notification_pb2 import MessagingRequest
from .grpc_lib.darsman_notification_pb2_grpc import (
    MessagingControllerStub,
)

CONTROLLER_STUB = MessagingControllerStub()
REQUEST_MESSAGE = MessagingRequest()
RPC_METHOD_NAME = "SendMessage"
SUCCESS_CODE = "0"


class NotificationTypes:
    SMS_NOTIFICATION = "sms"
    EMAIL_NOTIFICATION = "email"


class EventProTypes:
    LOGIN = "login"
    REGISTER = "register"
    WELCOME = "welcome"
    DISCOUNT_1MO_AFTER_REG = "discount_1mo_after_reg"
    HAPPY_BIRTHDAY = "hbd"
    CERTIFICATE_SUBMIT = "cert_submit"
    CERTIFICATE_EDIT = "cert_edit"
    MASTER_COURSE_INFORM = "edu_sup_inform"
    INITIAL_REGISTER = "init_course_reg"
    ROAD_MAP = "roadmap"
    CONSULT_REQUEST = "success_consult_req"
    DISCOUNT = "discount"
    MASTER_COURSE_REGISTER_FORM = "master_course_register_form"



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from . import darsman_notification_pb2


class MessagingControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.SendMessage = self.channel.unary_unary(
                "/config.darsman_notification.MessagingController/SendMessage",
                request_serializer=darsman_notification_pb2.MessagingRequest.SerializeToString,
                response_deserializer=darsman_notification_pb2.MessagingResponse.FromString,
            )


class MessagingControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SendMessage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_MessagingControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "SendMessage": grpc.unary_unary_rpc_method_handler(
            servicer.SendMessage,
            request_deserializer=darsman_notification_pb2.MessagingRequest.FromString,
            response_serializer=darsman_notification_pb2.MessagingResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.darsman_notification.MessagingController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class MessagingController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SendMessage(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.darsman_notification.MessagingController/SendMessage",
            darsman_notification_pb2.MessagingRequest.SerializeToString,
            darsman_notification_pb2.MessagingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



import logging
from typing import ClassVar

from google.protobuf.struct_pb2 import Struct
from grpc.aio import insecure_channel
from pydantic import (
    BaseModel,
    EmailStr,
    HttpUrl,
    StringConstraints,
    validate_call,
)
from typing_extensions import Annotated

from .api_constants import (
    CONTROLLER_STUB,
    REQUEST_MESSAGE,
    RPC_METHOD_NAME,
    SUCCESS_CODE,
    EventProTypes,
    NotificationTypes,
)

logger = logging.getLogger(__name__)

IranianPhoneNumber = Annotated[
    str,
    StringConstraints(
        strip_whitespace=True, pattern=r"^(?:\+98|098|98|0098|)0?9\d{9}$"
    ),
]


class ResponseCodeError(Exception):
    pass


class ValidationError(Exception):
    pass


class Notify(BaseModel):
    """
    Configures the host and port of the notification server.

    Parameters:
        host (str): The host address of the notification server.
        port (int): The port number of the notification server.
        sender_unit (str): The name of the service utilizing this library.
    """

    host: str = "127.0.0.1"
    port: str | int = "50051"
    sender_unit: str = "notify client"

    __instance: ClassVar = None

    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    @staticmethod
    def determine_send_type(recipient):
        """Determine the send_type based on recipient."""
        # Check if recipient is an email or phone number
        if "@" in recipient:
            send_type = NotificationTypes.EMAIL_NOTIFICATION
        else:
            send_type = NotificationTypes.SMS_NOTIFICATION

        return send_type

    async def perform_send(self, send_type, data, pro_type, recipient, extra_data):
        """Perform the actual sending of the `data`."""
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            CONTROLLER_STUB.channel = channel
            CONTROLLER_STUB.ready()
            RPCMethod = getattr(CONTROLLER_STUB, RPC_METHOD_NAME, None)
            if RPCMethod:
                # Convert `data` to proto struct
                data_struct = Struct()
                data_struct.update(data)

                # Assign attributes to request message
                REQUEST_MESSAGE.send_type = send_type
                REQUEST_MESSAGE.sender_unit = self.sender_unit
                REQUEST_MESSAGE.data.update(data_struct)
                REQUEST_MESSAGE.pro_type = pro_type
                REQUEST_MESSAGE.recipient = recipient
                REQUEST_MESSAGE.extra_data = extra_data

                # Call rpc method
                response = await RPCMethod(REQUEST_MESSAGE)

                # Clear the request message for future use
                REQUEST_MESSAGE.Clear()

                # Raise error if response is not successful
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)

                return response
            else:
                raise ValidationError("Invalid RPC method name.")

    @validate_call
    async def send_message(
        self,
        *,
        data: dict,
        recipient: IranianPhoneNumber | EmailStr,
        extra_data: str,
        pro_type: str,
    ):
        """Send a particular message containing `data` to the recipient.

        If the `recipient` is a valid email/mobile number, an email/sms will be sent."""

        # Determine the send type based on the recipient
        send_type = self.determine_send_type(recipient)

        # Perform send
        return await self.perform_send(send_type, data, pro_type, recipient, extra_data)

    @validate_call
    async def send_login(self, *, otp: str, recipient: IranianPhoneNumber | EmailStr):
        logger.debug(f"sending login otp to {recipient}")
        data = {"otp": otp}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Login OTP To User",
            pro_type=EventProTypes.LOGIN,
        )

    @validate_call
    async def send_register(
        self, *, otp: str, recipient: IranianPhoneNumber | EmailStr
    ):
        logger.debug(f"sending register otp to {recipient}")
        data = {"otp": otp}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Register OTP To User",
            pro_type=EventProTypes.REGISTER,
        )

    @validate_call
    async def send_welcome(
        self,
        *,
        recipient: IranianPhoneNumber | EmailStr,
        user_name: str,
        discount_code: str,
        discount_percentage: int,
    ):
        logger.debug(f"sending welcome notify to {recipient}")
        data = {
            "user": user_name,
            "discount_code": discount_code,
            "discount_percent": discount_percentage,
        }
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Welcome Notify to User",
            pro_type=EventProTypes.WELCOME,
        )

    @validate_call
    async def send_discount_1mo_after_reg(
        self,
        *,
        recipient: IranianPhoneNumber | EmailStr,
        user_name: str,
        discount_code: str,
        validity_duration: str,
    ):
        logger.debug(f"sending discount 1 month after registration to {recipient}")
        data = {
            "user": user_name,
            "discount_code": discount_code,
            "validity_duration": validity_duration,
        }
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Discount 1 month after registration to User",
            pro_type=EventProTypes.DISCOUNT_1MO_AFTER_REG,
        )

    @validate_call
    async def send_hbd(
        self,
        *,
        recipient: IranianPhoneNumber | EmailStr,
        user_name: str,
        discount_code: str,
        discount_percentage: int,
    ):
        logger.debug(f"sending happy birthday to {recipient}")
        data = {
            "user": user_name,
            "discount_code": discount_code,
            "discount_percent": discount_percentage,
        }
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Happy birthday to User",
            pro_type=EventProTypes.HAPPY_BIRTHDAY,
        )

    @validate_call
    async def send_cert_submit(
        self, *, recipient: IranianPhoneNumber | EmailStr, user_name: str, date: str
    ):
        logger.debug(f"sending certificate submit to {recipient}")
        data = {"user": user_name, "date": date}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending certificate submit to User",
            pro_type=EventProTypes.CERTIFICATE_SUBMIT,
        )

    @validate_call
    async def send_cert_edit(
        self, *, recipient: IranianPhoneNumber | EmailStr, user_name: str
    ):
        logger.debug(f"sending certificate edit to {recipient}")
        data = {"user": user_name}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending certificate edit to User",
            pro_type=EventProTypes.CERTIFICATE_EDIT,
        )

    @validate_call
    async def send_edu_sup_inform(
        self, *, recipient: IranianPhoneNumber | EmailStr, user_name: str, course: str
    ):
        logger.debug(f"sending master course to {recipient}")
        data = {"user": user_name, "course": course}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Master Course to User",
            pro_type=EventProTypes.MASTER_COURSE_INFORM,
        )

    @validate_call
    async def send_init_course_reg(
        self,
        *,
        recipient: IranianPhoneNumber | EmailStr,
        user_name: str,
        support_phone_number: str | int,
    ):
        logger.debug(f"sending init course register to {recipient}")
        data = {"user": user_name, "support_phone_number": support_phone_number}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Init Course Register to User",
            pro_type=EventProTypes.INITIAL_REGISTER,
        )

    @validate_call
    async def send_roadmap(
        self, *, recipient: IranianPhoneNumber | EmailStr, link: HttpUrl, course: str
    ):
        logger.debug(f"sending roadmap to {recipient}")
        data = {"link": link, "course": course}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending RoadMap to User",
            pro_type=EventProTypes.ROAD_MAP,
        )

    @validate_call
    async def send_success_consult_req(
        self, *, recipient: IranianPhoneNumber | EmailStr, user_name: str, course: str
    ):
        logger.debug(f"sending successful consult request notify to {recipient}")
        data = {"user": user_name, "course": course}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Successful Consult Request Notify to User",
            pro_type=EventProTypes.CONSULT_REQUEST,
        )

    @validate_call
    async def send_discount(
        self,
        *,
        recipient: IranianPhoneNumber | EmailStr,
        user_name: str,
        course: str,
        support_phone_number: str,
        discount_code: str,
        discount_percentage: int,
    ):
        logger.debug(f"sending dicount notify to {recipient}")
        data = {
            "user": user_name,
            "course": course,
            "support_phone_number": support_phone_number,
            "discount_code": discount_code,
            "discount_percent": discount_percentage,
        }
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending Discount Notify to User",
            pro_type=EventProTypes.DISCOUNT,
        )

    @validate_call
    async def send_master_course_reg_form(
        self, *, recipient: IranianPhoneNumber | EmailStr, link: HttpUrl
    ):
        logger.debug(f"sending master course register form to {recipient}")
        data = {"link": str(link)}
        return await self.send_message(
            data=data,
            recipient=recipient,
            extra_data="Sending master course register form to User",
            pro_type=EventProTypes.MASTER_COURSE_REGISTER_FORM,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: darsman_notification/grpc/darsman_notification.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n4darsman_notification/grpc/darsman_notification.proto\x12\x1b\x63onfig.darsman_notification\x1a\x1cgoogle/protobuf/struct.proto"\x9a\x01\n\x10MessagingRequest\x12\x13\n\x0bsender_unit\x18\x01 \x01(\t\x12\x11\n\trecipient\x18\x03 \x01(\t\x12\x12\n\nextra_data\x18\x04 \x01(\t\x12\x10\n\x08pro_type\x18\x05 \x01(\t\x12\x11\n\tsend_type\x18\x06 \x01(\t\x12%\n\x04\x64\x61ta\x18\x07 \x01(\x0b\x32\x17.google.protobuf.Struct"@\n\x11MessagingResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t2\x85\x01\n\x13MessagingController\x12n\n\x0bSendMessage\x12-.config.darsman_notification.MessagingRequest\x1a..config.darsman_notification.MessagingResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "darsman_notification.grpc.darsman_notification_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_MESSAGINGREQUEST"]._serialized_start = 116
    _globals["_MESSAGINGREQUEST"]._serialized_end = 270
    _globals["_MESSAGINGRESPONSE"]._serialized_start = 272
    _globals["_MESSAGINGRESPONSE"]._serialized_end = 336
    _globals["_MESSAGINGCONTROLLER"]._serialized_start = 339
    _globals["_MESSAGINGCONTROLLER"]._serialized_end = 472
# @@protoc_insertion_point(module_scope)
